Height

Struct Height 

pub struct Height(/* private fields */);
An absolute block height, guaranteed to always contain a valid height value.

Implementations

Source

impl Height

pub const ZERO: Self

Absolute block height 0, the genesis block.

pub const MIN: Self = Self::ZERO

The minimum absolute block height (0), the genesis block.

pub const MAX: Self

The maximum absolute block height.

pub fn from_hex(s: &str) -> Result<Self, ParseHeightError>

Constructs a new Height from a hex string.

The input string may or may not contain a typical hex prefix e.g., 0x.

Errors

If the input string is not a valid hex representation of a block height.

pub const fn from_u32(n: u32) -> Result<Self, ConversionError>

Constructs a new block height directly from a u32 value.

Errors

If n does not represent a block height within the valid range for a locktime: [0, 499_999_999].

Examples
use bitcoin_units::locktime::absolute;

let h: u32 = 741521;
let height = absolute::Height::from_u32(h)?;
assert_eq!(height.to_u32(), h);
pub const fn to_u32(self) -> u32

Converts this Height to a raw u32 value.

Examples
use bitcoin_units::locktime::absolute;

assert_eq!(absolute::Height::MAX.to_u32(), 499_999_999);
pub fn is_satisfied_by(self, height: Self) -> bool

Returns true if a transaction with this locktime can be included in the next block.

self is value of the LockTime and if height is the current chain tip then a transaction with this lock can be broadcast for inclusion in the next block.