#[non_exhaustive]pub enum Denomination {
Bitcoin,
CentiBitcoin,
MilliBitcoin,
MicroBitcoin,
Bit,
Satoshi,
}Expand description
A set of denominations in which amounts can be expressed.
Accepted Denominations
All upper or lower case, excluding SI prefixes c, m and u (or µ) which must be lower case.
- Singular: BTC, cBTC, mBTC, uBTC
- Plural or singular: sat, satoshi, bit
Note
Due to ambiguity between mega and milli we prohibit usage of leading capital ‘M’. It is more important to protect users from incorrectly using a capital M to mean milli than to allow Megabitcoin which is not a realistic denomination, and Megasatoshi which is equivalent to cBTC which is allowed.
Examples
let equal = [
("1 BTC", 100_000_000),
("1 cBTC", 1_000_000),
("1 mBTC", 100_000),
("1 uBTC", 100),
("1 bit", 100),
("1 sat", 1),
];
for (string, sats) in equal {
assert_eq!(
string.parse::<Amount>().expect("valid bitcoin amount string"),
Amount::from_sat(sats)?,
)
}Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Bitcoin
BTC (1 BTC = 100,000,000 satoshi).
CentiBitcoin
cBTC (1 cBTC = 1,000,000 satoshi).
MilliBitcoin
mBTC (1 mBTC = 100,000 satoshi).
MicroBitcoin
µBTC (1 µBTC = 100 satoshi).
Bit
bits (bits = µBTC).
Satoshi
satoshi (1 BTC = 100,000,000 satoshi).
Implementations