Module error

Module error 

Error types for block height and interval types.

Structs

BlockHeightDecoderError
An error consensus decoding an BlockHeight.
TooBigForRelativeHeightError
Error returned when the block interval is too big to be used as a relative lock time.