NumOpResult

Enum NumOpResult 

pub enum NumOpResult<T> {
    Valid(T),
    Error(NumOpError),
}
Result of a mathematical operation on two numeric types.

In order to prevent overflow we provide a custom result type that is similar to the normal core::result::Result but implements mathematical operations (e.g. core::ops::Add) so that math operations can be chained ergonomically. This is very similar to how NaN works.

NumOpResult is a monadic type that contains Valid and Error (similar to Ok and Err). It supports a subset of functions similar to Result (e.g. unwrap).

Examples

The NumOpResult type provides protection against overflow and div-by-zero.

Overflow protection

// Example UTXO value.
let a1 = Amount::from_sat(1_000_000)?;
// And another value from some other UTXO.
let a2 = Amount::from_sat(765_432)?;
// Just an example (typically one would calculate fee using weight and fee rate).
let fee = Amount::from_sat(1_00)?;
// The amount we want to send.
let spend = Amount::from_sat(1_200_000)?;

// We can error if the change calculation overflows.
//
// For example if the `spend` value comes from the user and the `change` value is later
// used then overflow here could be an attack vector.
let _change = (a1 + a2 - spend - fee).into_result().expect("handle this error");

// Or if we control all the values and know they are sane we can just `unwrap`.
let _change = (a1 + a2 - spend - fee).unwrap();
// `NumOpResult` also implements `expect`.
let _change = (a1 + a2 - spend - fee).expect("we know values don't overflow");

Divide-by-zero (overflow in Div or Rem)

In some instances one may wish to differentiate div-by-zero from overflow.

// Two amounts that will be added to calculate the max fee.
let a = Amount::from_sat(123).expect("valid amount");
let b = Amount::from_sat(467).expect("valid amount");
// Fee rate for transaction.
let fee_rate = FeeRate::from_sat_per_vb(1);

// Somewhat contrived example to show addition operator chained with division.
let max_fee = a + b;
let _fee = match max_fee / fee_rate {
    NumOpResult::Valid(fee) => fee,
    NumOpResult::Error(e) if e.is_div_by_zero() => {
        // Do something when div by zero.
        return Err(e);
    },
    NumOpResult::Error(e) => {
        // We separate div-by-zero from overflow in case it needs to be handled separately.
        //
        // This branch could be hit since `max_fee` came from some previous calculation. And if
        // an input to that calculation was from the user then overflow could be an attack vector.
        return Err(e);
    }
};

Variants



Valid(T)

Result of a successful mathematical operation.



Error(NumOpError)

Result of an unsuccessful mathematical operation.

Implementations

Source

impl<T> NumOpResult<T>

pub fn map<U, F: FnOnce(T) -> U>(self, op: F) -> NumOpResult<U>

Maps a NumOpResult<T> to NumOpResult<U> by applying a function to a contained NumOpResult::Valid value, leaving a NumOpResult::Error value untouched.

Source

impl<T: Debug> NumOpResult<T>

pub fn expect(self, msg: &str) -> T

Returns the contained valid numeric type, consuming self.

Panics

Panics with msg if the numeric result is an Error.

pub fn unwrap(self) -> T

Returns the contained valid numeric type, consuming self.

Panics

Panics if the numeric result is an Error.

pub fn unwrap_err(self) -> NumOpError

Returns the contained error, consuming self.

Panics

Panics if the numeric result is valid.

pub fn unwrap_or(self, default: T) -> T

Returns the contained Some value or a provided default.

Arguments passed to unwrap_or are eagerly evaluated; if you are passing the result of a function call, it is recommended to use unwrap_or_else, which is lazily evaluated.

pub fn unwrap_or_else<F>(self, f: F) -> T
where F: FnOnce() -> T,

Returns the contained Some value or computes it from a closure.

pub fn ok(self) -> Option<T>

Converts this NumOpResult to an Option<T>.

pub fn into_result(self) -> Result<T, NumOpError>

Converts this NumOpResult to a Result<T, NumOpError>.

pub fn and_then<F>(self, op: F) -> Self
where F: FnOnce(T) -> Self,

Calls op if the numeric result is Valid, otherwise returns the Error value of self.

pub fn is_valid(&self) -> bool

Returns true if the numeric result is valid.

pub fn is_error(&self) -> bool

Returns true if the numeric result is invalid.