Module error

Module error 

Error types for integer parsing utilities.

Structs

ParseIntError
Error with rich context returned when a string can’t be parsed as an integer.
PrefixedHexError
Error returned when parsing an integer from a hex string that is supposed to contain a prefix.
UnprefixedHexError
Error returned when parsing an integer from a hex string that is not supposed to contain a prefix.