BlockHeight

Struct BlockHeight 

pub struct BlockHeight(/* private fields */);
A block height. Zero denotes the genesis block.

This type is not meant for constructing height based timelocks. It is a general purpose blockheight abstraction. For locktimes please see locktime::absolute::Height.

This is a thin wrapper around a u32 that may take on all values of a u32.

Implementations

Source

impl BlockHeight

pub const ZERO: Self

Block height 0, the genesis block.

pub const MIN: Self = Self::ZERO

The minimum block height (0), the genesis block.

pub const MAX: Self

The maximum block height.

pub const fn from_u32(inner: u32) -> Self

Constructs a new block height from a u32.

pub const fn to_u32(self) -> u32

Returns block height as a u32.

pub fn checked_sub(self, other: Self) -> Option<BlockHeightInterval>

Attempt to subtract two BlockHeights, returning None if overflow occurred.

pub fn checked_add(self, other: BlockHeightInterval) -> Option<Self>

Attempt to add an interval to this BlockHeight, returning None if overflow occurred.

pub const fn saturating_add(self, rhs: BlockHeightInterval) -> Self

Saturating integer addition.

Computes self + rhs, saturating at BlockHeight::MAX instead of overflowing.

pub const fn saturating_sub(self, rhs: BlockHeightInterval) -> Self

Saturating integer subtraction.

Computes self - rhs, saturating at BlockHeight::MIN instead of overflowing.