BlockHeightInterval

Struct BlockHeightInterval 

pub struct BlockHeightInterval(/* private fields */);
An unsigned block interval.

Block interval is an integer type representing a difference between the heights of two blocks.

This type is not meant for constructing relative height based timelocks. It is a general purpose block interval abstraction. For locktimes please see locktime::relative::NumberOfBlocks.

Implementations

Source

impl BlockHeightInterval

pub const ZERO: Self

Block interval 0.

pub const MIN: Self = Self::ZERO

The minimum block interval, equivalent to Self::ZERO.

pub const MAX: Self

The maximum block interval.

pub const fn from_u32(inner: u32) -> Self

Constructs a new block interval from a u32.

pub const fn to_u32(self) -> u32

Returns block interval as a u32.

pub fn checked_sub(self, other: Self) -> Option<Self>

Attempt to subtract two BlockHeightIntervals, returning None if overflow occurred.

pub fn checked_add(self, other: Self) -> Option<Self>

Attempt to add two BlockHeightIntervals, returning None if overflow occurred.