BlockMtp

Struct BlockMtp 

pub struct BlockMtp(/* private fields */);
The median timestamp of 11 consecutive blocks.

This type is not meant for constructing time-based timelocks. It is a general purpose MTP abstraction. For locktimes please see locktime::absolute::MedianTimePast.

This is a thin wrapper around a u32 that may take on all values of a u32.

Implementations

Source

impl BlockMtp

pub const ZERO: Self

Block MTP 0.

Since MTP is a timestamp, 0 is before Bitcoin was invented. This const may still be useful for some use cases e.g., folding a sum of intervals.

pub const MIN: Self = Self::ZERO

The minimum block MTP, equivalent to Self::ZERO.

pub const MAX: Self

The maximum block MTP.

pub const fn from_u32(inner: u32) -> Self

Constructs a new block MTP from a u32.

pub const fn to_u32(self) -> u32

Returns block MTP as a u32.

pub fn new(timestamps: [BlockTime; 11]) -> Self

Constructs a BlockMtp by computing the median‐time‐past from the last 11 block timestamps

Because block timestamps are not monotonic, this function internally sorts them; it is therefore not important what order they appear in the array; use whatever is most convenient.

pub fn checked_sub(self, other: Self) -> Option<BlockMtpInterval>

Attempt to subtract two BlockMtps, returning None if overflow occurred.

pub fn checked_add(self, other: BlockMtpInterval) -> Option<Self>

Attempt to add an interval to this BlockMtp, returning None if overflow occurred.