BlockMtpInterval

Struct BlockMtpInterval 

pub struct BlockMtpInterval(/* private fields */);
An unsigned difference between two BlockMtps.

This type is not meant for constructing time-based timelocks. It is a general purpose MTP abstraction. For locktimes please see locktime::relative::NumberOf512Seconds.

This is a thin wrapper around a u32 that may take on all values of a u32.

Implementations

Source

impl BlockMtpInterval

pub const ZERO: Self

Block MTP interval 0.

pub const MIN: Self = Self::ZERO

The minimum block MTP interval, equivalent to Self::ZERO.

pub const MAX: Self

The maximum block MTP interval.

pub const fn from_u32(inner: u32) -> Self

Constructs a new block MTP interval from a u32.

pub const fn to_u32(self) -> u32

Returns block MTP interval as a u32.

pub const fn to_relative_mtp_interval_floor( self, ) -> Result<NumberOf512Seconds, TimeOverflowError>

Converts a BlockMtpInterval to a locktime::relative::NumberOf512Seconds, rounding down.

Relative timelock MTP intervals have a resolution of 512 seconds, while BlockMtpInterval, like all block timestamp types, has a one-second resolution.

Errors

Errors if the MTP is out-of-range (in excess of 512 times u16::MAX seconds, or about 388 days) for a time-based relative locktime.

pub const fn to_relative_mtp_interval_ceil( self, ) -> Result<NumberOf512Seconds, TimeOverflowError>

Converts a BlockMtpInterval to a locktime::relative::NumberOf512Seconds, rounding up.

Relative timelock MTP intervals have a resolution of 512 seconds, while BlockMtpInterval, like all block timestamp types, has a one-second resolution.

Errors

Errors if the MTP is out-of-range (in excess of 512 times u16::MAX seconds, or about 388 days) for a time-based relative locktime.

pub fn checked_sub(self, other: Self) -> Option<Self>

Attempt to subtract two BlockMtpIntervals, returning None if overflow occurred.

pub fn checked_add(self, other: Self) -> Option<Self>

Attempt to add two BlockMtpIntervals, returning None if overflow occurred.