SignedAmount

Struct SignedAmount 

pub struct SignedAmount(/* private fields */);
A signed amount.

The SignedAmount type can be used to express Bitcoin amounts that support arithmetic and conversion to various denominations. The SignedAmount type does not implement serde traits but we do provide modules for serializing as satoshis or bitcoin.

Warning!

This type implements several arithmetic operations from core::ops. To prevent errors due to an overflow when using these operations, it is advised to instead use the checked arithmetic methods whose names start with checked_. The operations from core::ops that SignedAmount implements will panic when an overflow occurs.

Examples

use serde::{Serialize, Deserialize};
use bitcoin_units::SignedAmount;

#[derive(Serialize, Deserialize)]
struct Foo {
    // If you are using `rust-bitcoin` then `bitcoin::amount::serde::as_sat` also works.
    #[serde(with = "bitcoin_units::amount::serde::as_sat")]  // Also `serde::as_btc`.
    amount: SignedAmount,
}

Implementations

Source

impl SignedAmount

pub const MAX: Self

The maximum value of an amount.

pub const MIN: Self

The minimum value of an amount.

pub const fn to_sat(self) -> i64

Gets the number of satoshis in this SignedAmount.

Examples
assert_eq!(SignedAmount::ONE_BTC.to_sat(), 100_000_000);
pub const fn from_sat(satoshi: i64) -> Result<Self, OutOfRangeError>

Constructs a new SignedAmount from the given number of satoshis.

Errors

If satoshi is outside of valid range (see Self::MAX_MONEY).

Examples
let amount = SignedAmount::from_sat(sat)?;
assert_eq!(amount.to_sat(), sat);
Source

impl SignedAmount

pub const ZERO: Self

The zero amount.

pub const ONE_SAT: Self

Exactly one satoshi.

pub const ONE_BTC: Self

Exactly one bitcoin.

pub const FIFTY_BTC: Self

Exactly fifty bitcoin.

pub const MAX_MONEY: Self = Self::MAX

The maximum value allowed as an amount. Useful for sanity checking.

pub const fn from_sat_i32(satoshi: i32) -> Self

Constructs a new SignedAmount with satoshi precision and the given number of satoshis.

Accepts an i32 which is guaranteed to be in range for the type, but which can only represent roughly -21.47 to 21.47 BTC.

pub fn from_btc(btc: f64) -> Result<Self, ParseAmountError>

Converts from a value expressing a decimal number of bitcoin to a SignedAmount.

Errors

If the amount is too big (positive or negative) or too precise.

Please be aware of the risk of using floating-point numbers.

Examples
let amount = SignedAmount::from_btc(-0.01)?;
assert_eq!(amount.to_sat(), -1_000_000);
pub fn from_int_btc<T: Into<i16>>(whole_bitcoin: T) -> Self

Converts from a value expressing a whole number of bitcoin to a SignedAmount.

pub const fn from_btc_i16(whole_bitcoin: i16) -> Self

Converts from a value expressing a whole number of bitcoin to a SignedAmount in const context.

pub fn from_str_in( s: &str, denom: Denomination, ) -> Result<Self, ParseAmountError>

Parses a decimal string as a value in the given Denomination.

Note: This only parses the value string. If you want to parse a string containing the value with denomination, use FromStr.

Errors

If the amount is too big (positive or negative) or too precise.

pub fn from_str_with_denomination(s: &str) -> Result<Self, ParseError>

Parses amounts with denomination suffix as produced by Self::to_string_with_denomination or with fmt::Display.

If you want to parse only the amount without the denomination, use Self::from_str_in.

Errors

If the amount is too big (positive or negative) or too precise.

Examples
let amount = SignedAmount::from_str_with_denomination("0.1 BTC")?;
assert_eq!(amount, SignedAmount::from_sat(10_000_000)?);
pub fn to_float_in(self, denom: Denomination) -> f64

Expresses this SignedAmount as a floating-point value in the given Denomination.

Please be aware of the risk of using floating-point numbers.

Examples
let amount = SignedAmount::from_sat(100_000)?;
assert_eq!(amount.to_float_in(Denomination::Bitcoin), 0.001);
pub fn to_btc(self) -> f64

Expresses this SignedAmount as a floating-point value in Bitcoin.

Please be aware of the risk of using floating-point numbers.

Examples
let amount = SignedAmount::from_sat(100_000)?;
assert_eq!(amount.to_btc(), amount.to_float_in(Denomination::Bitcoin));
pub fn from_float_in( value: f64, denom: Denomination, ) -> Result<Self, ParseAmountError>

Converts this SignedAmount in floating-point notation in the given Denomination.

Errors

If the amount is too big (positive or negative) or too precise.

Please be aware of the risk of using floating-point numbers.

pub fn display_in(self, denomination: Denomination) -> Display

Constructs a new object that implements fmt::Display in the given Denomination.

This function is useful if you do not wish to allocate. See also Self::to_string_in.

Examples
let amount = SignedAmount::from_sat(10_000_000)?;
let mut output = String::new();
let _ = write!(&mut output, "{}", amount.display_in(Denomination::Bitcoin));
assert_eq!(output, "0.1");
pub fn display_dynamic(self) -> Display

Constructs a new object that implements fmt::Display dynamically selecting Denomination.

This will use BTC for values greater than or equal to 1 BTC and satoshis otherwise. To avoid confusion the denomination is always shown.

pub fn to_string_in(self, denom: Denomination) -> String

Returns a formatted string representing this SignedAmount in the given Denomination.

Returned string does not include the denomination.

Examples
let amount = SignedAmount::from_sat(10_000_000)?;
assert_eq!(amount.to_string_in(Denomination::Bitcoin), "0.1");
pub fn to_string_with_denomination(self, denom: Denomination) -> String

Returns a formatted string representing this SignedAmount in the given Denomination, suffixed with the abbreviation for the denomination.

Examples
let amount = SignedAmount::from_sat(10_000_000)?;
assert_eq!(amount.to_string_with_denomination(Denomination::Bitcoin), "0.1 BTC");
pub const fn abs(self) -> Self

Gets the absolute value of this SignedAmount.

This function never overflows or panics, unlike i64::abs().

pub fn unsigned_abs(self) -> Amount

Gets the absolute value of this SignedAmount returning Amount.

pub fn signum(self) -> i64

Returns a number representing sign of this SignedAmount.

  • 0 if the amount is zero
  • 1 if the amount is positive
  • -1 if the amount is negative
pub fn is_positive(self) -> bool

Checks if this SignedAmount is positive.

Returns true if this SignedAmount is positive and false if this SignedAmount is zero or negative.

pub fn is_negative(self) -> bool

Checks if this SignedAmount is negative.

Returns true if this SignedAmount is negative and false if this SignedAmount is zero or positive.

pub const fn checked_abs(self) -> Option<Self>

👎Deprecated since 1.0.0-rc.0: Never returns none, use abs() instead

Returns the absolute value of this SignedAmount.

Consider using unsigned_abs which is often more practical.

Returns None if overflow occurred. (self == i64::MIN)

pub const fn checked_add(self, rhs: Self) -> Option<Self>

Checked addition.

Returns None if the sum is above SignedAmount::MAX or below SignedAmount::MIN.

pub const fn checked_sub(self, rhs: Self) -> Option<Self>

Checked subtraction.

Returns None if the difference is above SignedAmount::MAX or below SignedAmount::MIN.

pub const fn checked_mul(self, rhs: i64) -> Option<Self>

Checked multiplication.

Returns None if the product is above SignedAmount::MAX or below SignedAmount::MIN.

pub const fn checked_div(self, rhs: i64) -> Option<Self>

Checked integer division.

Be aware that integer division loses the remainder if no exact division can be made.

Returns None if overflow occurred.

pub const fn checked_rem(self, rhs: i64) -> Option<Self>

Checked remainder.

Returns None if overflow occurred.

pub fn positive_sub(self, rhs: Self) -> Option<Self>

Subtraction that doesn’t allow negative SignedAmounts.

Returns None if either self, rhs or the result is strictly negative.

pub fn to_unsigned(self) -> Result<Amount, OutOfRangeError>

Converts to an unsigned amount.

Errors

If the amount is negative.