Weight

Struct Weight 

pub struct Weight(/* private fields */);
The weight of a transaction or block.

This is an integer newtype representing weight in weight units. It provides protection against mixing up the types, conversion functions, and basic formatting.

Implementations

Source

impl Weight

pub const fn from_wu(wu: u64) -> Self

Constructs a new Weight from weight units.

pub const fn to_wu(self) -> u64

Returns raw weight units.

Can be used instead of into() to avoid inference issues.

Source

impl Weight

pub const ZERO: Self

Zero weight units (wu).

Equivalent to MIN, may better express intent in some contexts.

pub const MIN: Self

Minimum possible value (0 wu).

Equivalent to ZERO, may better express intent in some contexts.

pub const MAX: Self

Maximum possible value.

pub const WITNESS_SCALE_FACTOR: u64

The factor that non-witness serialization data is multiplied by during weight calculation.

pub const MAX_BLOCK: Self

The maximum allowed weight for a block, see BIP-0141 (network rule).

pub const MIN_TRANSACTION: Self

The minimum transaction weight for a valid serialized transaction.

pub const fn from_kwu(wu: u64) -> Option<Self>

Constructs a new Weight from kilo weight units returning None if an overflow occurred.

pub const fn from_vb(vb: u64) -> Option<Self>

Constructs a new Weight from virtual bytes, returning None if an overflow occurred.

pub const fn from_vb_unwrap(vb: u64) -> Self

👎Deprecated since 1.0.0-rc.0: use from_vb_unchecked instead

Constructs a new Weight from virtual bytes panicking if an overflow occurred.

Panics

If the conversion from virtual bytes overflows.

pub const fn from_vb_unchecked(vb: u64) -> Self

Constructs a new Weight from virtual bytes without an overflow check.

pub const fn from_witness_data_size(witness_size: u64) -> Self

👎Deprecated since 1.0.0-rc.1: use from_wu instead

Constructs a new Weight from witness size.

pub const fn from_non_witness_data_size(non_witness_size: u64) -> Self

👎Deprecated since 1.0.0-rc.1: use from_vb or from_vb_unchecked instead

Constructs a new Weight from non-witness size.

Panics

If the conversion from virtual bytes overflows.

pub const fn to_kwu_floor(self) -> u64

Converts to kilo weight units rounding down.

pub const fn to_kwu_ceil(self) -> u64

Converts to kilo weight units rounding up.

pub const fn to_vbytes_floor(self) -> u64

Converts to vB rounding down.

pub const fn to_vbytes_ceil(self) -> u64

Converts to vB rounding up.

pub const fn checked_add(self, rhs: Self) -> Option<Self>

Checked addition.

Computes self + rhs returning None if an overflow occurred.

pub const fn checked_sub(self, rhs: Self) -> Option<Self>

Checked subtraction.

Computes self - rhs returning None if an overflow occurred.

pub const fn checked_mul(self, rhs: u64) -> Option<Self>

Checked multiplication.

Computes self * rhs returning None if an overflow occurred.

pub const fn checked_div(self, rhs: u64) -> Option<Self>

Checked division.

Computes self / rhs returning None if rhs == 0.

pub const fn mul_by_fee_rate(self, fee_rate: FeeRate) -> NumOpResult<Amount>

Checked fee rate multiplication.

Computes the absolute fee amount for a given FeeRate at this weight. When the resulting fee is a non-integer amount, the amount is rounded up, ensuring that the transaction fee is enough instead of falling short if rounded down.