pub struct Height(/* private fields */);Expand description
An absolute block height, guaranteed to always contain a valid height value.
Implementations
Sourceimpl Height
impl Height
Sourcepub fn from_hex(s: &str) -> Result<Self, ParseHeightError>
pub fn from_hex(s: &str) -> Result<Self, ParseHeightError>
Sourcepub const fn from_u32(n: u32) -> Result<Self, ConversionError>
pub const fn from_u32(n: u32) -> Result<Self, ConversionError>
Constructs a new block height directly from a u32 value.
Errors
If n does not represent a block height within the valid range for a locktime:
[0, 499_999_999].
Examples
use bitcoin_units::locktime::absolute;
let h: u32 = 741521;
let height = absolute::Height::from_u32(h)?;
assert_eq!(height.to_u32(), h);Sourcepub fn is_satisfied_by(self, height: Self) -> bool
pub fn is_satisfied_by(self, height: Self) -> bool
Returns true if a transaction with this locktime can be included in the next block.
self is value of the LockTime and if height is the current chain tip then
a transaction with this lock can be broadcast for inclusion in the next block.