Error types for integer parsing utilities.
Structs
- Parse
IntError - Error with rich context returned when a string can’t be parsed as an integer.
- Prefixed
HexError - Error returned when parsing an integer from a hex string that is supposed to contain a prefix.
- Unprefixed
HexError - Error returned when parsing an integer from a hex string that is not supposed to contain a prefix.