pub struct NumberOfBlocks(/* private fields */);Expand description
A relative lock time lock-by-height value.
Implementations
Sourceimpl NumberOfBlocks
impl NumberOfBlocks
Sourcepub const MIN: Self = Self::ZERO
pub const MIN: Self = Self::ZERO
The minimum relative block height (0), can be included in any block.
Sourcepub const fn from_height(blocks: u16) -> Self
pub const fn from_height(blocks: u16) -> Self
Constructs a new NumberOfBlocks using a count of blocks.
Sourcepub const fn to_height(self) -> u16
pub const fn to_height(self) -> u16
Express the NumberOfBlocks as a count of blocks.
Sourcepub const fn to_consensus_u32(self) -> u32
👎Deprecated since 1.0.0-rc.0: use LockTime::from followed by to_consensus_u32 instead
pub const fn to_consensus_u32(self) -> u32
LockTime::from followed by to_consensus_u32 insteadReturns the u32 value used to encode this locktime in an nSequence field or
argument to OP_CHECKSEQUENCEVERIFY.
Sourcepub fn is_satisfied_by(
self,
chain_tip: BlockHeight,
utxo_mined_at: BlockHeight,
) -> Result<bool, InvalidHeightError>
pub fn is_satisfied_by( self, chain_tip: BlockHeight, utxo_mined_at: BlockHeight, ) -> Result<bool, InvalidHeightError>
Returns true if an output locked by height can be spent in the next block.
Errors
If chain_tip as not after utxo_mined_at i.e., if you get the args mixed up.