pub struct BlockHeight(/* private fields */);Expand description
A block height. Zero denotes the genesis block.
This type is not meant for constructing height based timelocks. It is a general purpose
blockheight abstraction. For locktimes please see locktime::absolute::Height.
This is a thin wrapper around a u32 that may take on all values of a u32.
Implementations
Sourceimpl BlockHeight
impl BlockHeight
Sourcepub fn checked_sub(self, other: Self) -> Option<BlockHeightInterval>
pub fn checked_sub(self, other: Self) -> Option<BlockHeightInterval>
Attempt to subtract two BlockHeights, returning None if overflow occurred.
Sourcepub fn checked_add(self, other: BlockHeightInterval) -> Option<Self>
pub fn checked_add(self, other: BlockHeightInterval) -> Option<Self>
Attempt to add an interval to this BlockHeight, returning None if overflow occurred.
Sourcepub const fn saturating_add(self, rhs: BlockHeightInterval) -> Self
pub const fn saturating_add(self, rhs: BlockHeightInterval) -> Self
Saturating integer addition.
Computes self + rhs, saturating at BlockHeight::MAX instead of overflowing.
Sourcepub const fn saturating_sub(self, rhs: BlockHeightInterval) -> Self
pub const fn saturating_sub(self, rhs: BlockHeightInterval) -> Self
Saturating integer subtraction.
Computes self - rhs, saturating at BlockHeight::MIN instead of overflowing.