pub struct BlockHeightInterval(/* private fields */);Expand description
An unsigned block interval.
Block interval is an integer type representing a difference between the heights of two blocks.
This type is not meant for constructing relative height based timelocks. It is a general
purpose block interval abstraction. For locktimes please see locktime::relative::NumberOfBlocks.
Implementations
Sourceimpl BlockHeightInterval
impl BlockHeightInterval
Sourcepub fn checked_sub(self, other: Self) -> Option<Self>
pub fn checked_sub(self, other: Self) -> Option<Self>
Attempt to subtract two BlockHeightIntervals, returning None if overflow occurred.
Sourcepub fn checked_add(self, other: Self) -> Option<Self>
pub fn checked_add(self, other: Self) -> Option<Self>
Attempt to add two BlockHeightIntervals, returning None if overflow occurred.