pub struct BlockMtpInterval(/* private fields */);Expand description
An unsigned difference between two BlockMtps.
This type is not meant for constructing time-based timelocks. It is a general purpose
MTP abstraction. For locktimes please see locktime::relative::NumberOf512Seconds.
This is a thin wrapper around a u32 that may take on all values of a u32.
Implementations
Sourceimpl BlockMtpInterval
impl BlockMtpInterval
Sourcepub const fn to_relative_mtp_interval_floor(
self,
) -> Result<NumberOf512Seconds, TimeOverflowError>
pub const fn to_relative_mtp_interval_floor( self, ) -> Result<NumberOf512Seconds, TimeOverflowError>
Converts a BlockMtpInterval to a locktime::relative::NumberOf512Seconds, rounding down.
Relative timelock MTP intervals have a resolution of 512 seconds, while
BlockMtpInterval, like all block timestamp types, has a one-second resolution.
Errors
Errors if the MTP is out-of-range (in excess of 512 times u16::MAX seconds, or about
388 days) for a time-based relative locktime.
Sourcepub const fn to_relative_mtp_interval_ceil(
self,
) -> Result<NumberOf512Seconds, TimeOverflowError>
pub const fn to_relative_mtp_interval_ceil( self, ) -> Result<NumberOf512Seconds, TimeOverflowError>
Converts a BlockMtpInterval to a locktime::relative::NumberOf512Seconds, rounding up.
Relative timelock MTP intervals have a resolution of 512 seconds, while
BlockMtpInterval, like all block timestamp types, has a one-second resolution.
Errors
Errors if the MTP is out-of-range (in excess of 512 times u16::MAX seconds, or about
388 days) for a time-based relative locktime.
Sourcepub fn checked_sub(self, other: Self) -> Option<Self>
pub fn checked_sub(self, other: Self) -> Option<Self>
Attempt to subtract two BlockMtpIntervals, returning None if overflow occurred.
Sourcepub fn checked_add(self, other: Self) -> Option<Self>
pub fn checked_add(self, other: Self) -> Option<Self>
Attempt to add two BlockMtpIntervals, returning None if overflow occurred.