BlockTime

Struct BlockTime 

pub struct BlockTime(/* private fields */);
A Bitcoin block timestamp.

Each block contains a Unix time timestamp. In addition to serving as a source of variation for the block hash, they also make it more difficult for an adversary to manipulate the block chain.

A timestamp is accepted as valid if it is greater than the median timestamp of previous 11 blocks, and less than the network-adjusted time + 2 hours. “Network-adjusted time” is the median of the timestamps returned by all nodes connected to you. As a result block timestamps are not exactly accurate, and they do not need to be. Block times are accurate only to within an hour or two.

ref: https://en.bitcoin.it/wiki/Block_timestamp

Implementations

Source

impl BlockTime

pub const fn from_u32(t: u32) -> Self

Constructs a new BlockTime from an unsigned 32 bit integer value.

pub const fn to_u32(self) -> u32

Returns the inner u32 value.