pub struct CompactTarget(/* private fields */);Expand description
Encoding of 256-bit target as 32-bit float.
This is used to encode a target into the block header. Satoshi made this part of consensus code in the original version of Bitcoin, likely copying an idea from OpenSSL.
OpenSSL’s bignum (BN) type has an encoding, which is even called “compact” as in bitcoin, which is exactly this format.
Note on order/equality
Usage of the ordering and equality traits for this type may be surprising. Converting between
CompactTarget and Target is lossy in both directions (there are multiple CompactTarget
values that map to the same Target value). Ordering and equality for this type are defined in
terms of the underlying u32.
Implementations
Sourceimpl CompactTarget
impl CompactTarget
Sourcepub fn from_consensus(bits: u32) -> Self
pub fn from_consensus(bits: u32) -> Self
Constructs a new CompactTarget from a consensus encoded u32.
Sourcepub const fn to_consensus(self) -> u32
pub const fn to_consensus(self) -> u32
Returns the consensus encoded u32 representation of this CompactTarget.
Sourcepub fn to_hex(self) -> String
👎Deprecated since 1.0.0-rc.0: use format!("{var:x}") instead
pub fn to_hex(self) -> String
format!("{var:x}") insteadGets the hex representation of this CompactTarget.
Sourcepub fn from_hex(s: &str) -> Result<Self, PrefixedHexError>where
Self: Sized,
pub fn from_hex(s: &str) -> Result<Self, PrefixedHexError>where
Self: Sized,
Constructs a new CompactTarget from a prefixed hex string.
Errors
- If the input string does not contain a
0x(or0X) prefix. - If the input string is not a valid hex encoding of a
u32.
Sourcepub fn from_unprefixed_hex(s: &str) -> Result<Self, UnprefixedHexError>where
Self: Sized,
pub fn from_unprefixed_hex(s: &str) -> Result<Self, UnprefixedHexError>where
Self: Sized,
Constructs a new CompactTarget from an unprefixed hex string.
Errors
- If the input string contains a
0x(or0X) prefix. - If the input string is not a valid hex encoding of a
u32.