pub struct Weight(/* private fields */);Expand description
The weight of a transaction or block.
This is an integer newtype representing weight in weight units. It provides protection against mixing up the types, conversion functions, and basic formatting.
Implementations
Sourceimpl Weight
impl Weight
Sourcepub const ZERO: Self
pub const ZERO: Self
Zero weight units (wu).
Equivalent to MIN, may better express intent in some contexts.
Sourcepub const MIN: Self
pub const MIN: Self
Minimum possible value (0 wu).
Equivalent to ZERO, may better express intent in some contexts.
Sourcepub const WITNESS_SCALE_FACTOR: u64
pub const WITNESS_SCALE_FACTOR: u64
The factor that non-witness serialization data is multiplied by during weight calculation.
Sourcepub const MIN_TRANSACTION: Self
pub const MIN_TRANSACTION: Self
The minimum transaction weight for a valid serialized transaction.
Sourcepub const fn from_vb_unwrap(vb: u64) -> Self
👎Deprecated since 1.0.0-rc.0: use from_vb_unchecked instead
pub const fn from_vb_unwrap(vb: u64) -> Self
from_vb_unchecked insteadSourcepub const fn from_vb_unchecked(vb: u64) -> Self
pub const fn from_vb_unchecked(vb: u64) -> Self
Constructs a new Weight from virtual bytes without an overflow check.
Sourcepub const fn from_witness_data_size(witness_size: u64) -> Self
👎Deprecated since 1.0.0-rc.1: use from_wu instead
pub const fn from_witness_data_size(witness_size: u64) -> Self
from_wu insteadConstructs a new Weight from witness size.
Sourcepub const fn from_non_witness_data_size(non_witness_size: u64) -> Self
👎Deprecated since 1.0.0-rc.1: use from_vb or from_vb_unchecked instead
pub const fn from_non_witness_data_size(non_witness_size: u64) -> Self
from_vb or from_vb_unchecked insteadSourcepub const fn to_kwu_floor(self) -> u64
pub const fn to_kwu_floor(self) -> u64
Converts to kilo weight units rounding down.
Sourcepub const fn to_kwu_ceil(self) -> u64
pub const fn to_kwu_ceil(self) -> u64
Converts to kilo weight units rounding up.
Sourcepub const fn to_vbytes_floor(self) -> u64
pub const fn to_vbytes_floor(self) -> u64
Converts to vB rounding down.
Sourcepub const fn to_vbytes_ceil(self) -> u64
pub const fn to_vbytes_ceil(self) -> u64
Converts to vB rounding up.
Sourcepub const fn checked_add(self, rhs: Self) -> Option<Self>
pub const fn checked_add(self, rhs: Self) -> Option<Self>
Checked addition.
Computes self + rhs returning None if an overflow occurred.
Sourcepub const fn checked_sub(self, rhs: Self) -> Option<Self>
pub const fn checked_sub(self, rhs: Self) -> Option<Self>
Checked subtraction.
Computes self - rhs returning None if an overflow occurred.
Sourcepub const fn checked_mul(self, rhs: u64) -> Option<Self>
pub const fn checked_mul(self, rhs: u64) -> Option<Self>
Checked multiplication.
Computes self * rhs returning None if an overflow occurred.
Sourcepub const fn checked_div(self, rhs: u64) -> Option<Self>
pub const fn checked_div(self, rhs: u64) -> Option<Self>
Checked division.
Computes self / rhs returning None if rhs == 0.
Sourcepub const fn mul_by_fee_rate(self, fee_rate: FeeRate) -> NumOpResult<Amount>
pub const fn mul_by_fee_rate(self, fee_rate: FeeRate) -> NumOpResult<Amount>
Checked fee rate multiplication.
Computes the absolute fee amount for a given FeeRate at this weight. When the resulting
fee is a non-integer amount, the amount is rounded up, ensuring that the transaction fee is
enough instead of falling short if rounded down.